﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Grids;
using Infragistics.Documents.Excel;
using System.IO;
using Infragistics.Controls.Grids.Primitives;

namespace xamGrid_ExportToExcel
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
        }

        private void Btn_Export_Click(object sender, RoutedEventArgs e)
        {
            // Create workbook
            Workbook workbook1 = new Workbook();

            // Add worksheet
            Worksheet worksheet1 = workbook1.Worksheets.Add("Sample Worksheet1");

            WorksheetRow worksheetRow;
            Column gridColumn;
            WorksheetCell worksheetCell;

            // Keep track of which Row we're on.
            int rowIndex = 0;
            int indentLevel = 0;

            for (int i = 0; i < this.xamGrid1.Rows.Count; i++)
            {
                if (this.xamGrid1.Rows[i] is GroupByRow)
                    HandleGroupByRow(this.xamGrid1.Rows[i] as GroupByRow, worksheet1, ref rowIndex, indentLevel);
                else if (this.xamGrid1.Rows[i] is Row)
                {
                    Row gridRow = this.xamGrid1.Rows[i] as Row;

                    worksheetRow = worksheet1.Rows[gridRow.Index + 1];

                    for (int columnIndex = 0; columnIndex < this.xamGrid1.Columns.Count; columnIndex++)
                    {
                        // Set Excell cell value to xamGrid's cell value cell by cell
                        worksheetRow.Cells[columnIndex].Value = gridRow.Cells[columnIndex].Value;

                        /* Set header appearance and text */
                        if (gridRow.Index == 0)
                        {
                            worksheetCell = worksheet1.Rows[0].Cells[columnIndex];

                            gridColumn = this.xamGrid1.Columns[columnIndex] as Column;

                            //worksheet1.Columns[columnIndex].Width = (int)col.Width.Value;
                            // Set header cell color and and borders
                            worksheetCell.CellFormat.FillPatternForegroundColor = Colors.LightGray;
                            worksheetCell.CellFormat.LeftBorderColor = Color.FromArgb(0, 178, 178, 178);
                            worksheetCell.CellFormat.RightBorderColor = Color.FromArgb(0, 178, 178, 178);

                            // Set Excel header text to key of column
                            worksheetCell.Value = gridColumn.Key;
                        }
                    }
                }
            }

            // Create Save File Dialog
            SaveFileDialog savefileDialog = new SaveFileDialog { Filter = "Excel files|*.xls", DefaultExt = "xls" };

            // Display the dialog. If the user selects a file of the correct format 
            // and clicks Save in the dialog, save the file using the Save method of the Workbook object.
            if (savefileDialog.ShowDialog() == true)
            {
                Stream stream = savefileDialog.OpenFile();
                workbook1.Save(stream);
                stream.Close();
            }
        }

        public void HandleGroupByRow(GroupByRow groupedRow, Worksheet worksheet, ref int rowIndex, int indentLevel)
        {
            // The current row we're on.
            WorksheetRow worksheetRow = worksheet.Rows[rowIndex];

            // The header for these grouped rows.
            worksheetRow.Cells[indentLevel].Value = groupedRow.GroupByData.Value.ToString() + "(" + groupedRow.GroupByData.Count.ToString() + ")";

            // Move on to the next row.
            rowIndex++;

            // this is used to know when to place column headers.
            bool bFirstRow = true;

            for (int i = 0; i < groupedRow.Rows.Count; i++)
            {
                if (groupedRow.Rows[i] is GroupByRow)
                    HandleGroupByRow(groupedRow.Rows[i] as GroupByRow, worksheet, ref rowIndex, indentLevel + 1);
                else if (groupedRow.Rows[i] is Row)
                    HandleRow(groupedRow.Rows[i] as Row, worksheet, ref rowIndex, indentLevel, bFirstRow);

                bFirstRow = false;
            }
        }

        public void HandleRow(Row gridRow, Worksheet worksheet, ref int rowIndex, int indentLevel, bool headerRow)
        {
            // The current row we're on.
            WorksheetRow worksheetRow = worksheet.Rows[rowIndex];

            // First row needs column headers.
            if (headerRow)
            {
                for (int columnIndex = 0; columnIndex < this.xamGrid1.Columns.Count; columnIndex++)
                {
                    WorksheetCell worksheetCell = worksheet.Rows[rowIndex].Cells[indentLevel + columnIndex];

                    Column gridColumn = this.xamGrid1.Columns[columnIndex] as Column;

                    //worksheet1.Columns[columnIndex].Width = (int)col.Width.Value;
                    // Set header cell color and and borders
                    worksheetCell.CellFormat.FillPatternForegroundColor = Colors.LightGray;
                    worksheetCell.CellFormat.LeftBorderColor = Color.FromArgb(0, 178, 178, 178);
                    worksheetCell.CellFormat.RightBorderColor = Color.FromArgb(0, 178, 178, 178);

                    // Set Excel header text to key of column
                    worksheetCell.Value = gridColumn.Key;
                }

                // Next row.
                rowIndex++;

                // Update the worksheet row.
                worksheetRow = worksheet.Rows[rowIndex];
            }

            for (int columnIndex = 0; columnIndex < this.xamGrid1.Columns.Count; columnIndex++)
            {
                // Set Excel cell value to xamGrid's cell value cell by cell
                worksheetRow.Cells[indentLevel + columnIndex].Value = gridRow.Cells[columnIndex].Value;
            }
        }
    }
}
